# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-10-30 20:31
from __future__ import unicode_literals

from django.db import migrations, models

# Adds table-level permissions for executive group - 
# allows the addition and deletion of row-level permissions.
# Specifically, it's used to allow executives to expose/hide events
# from other groups (currently LV-EM, maybe the public in the future)

# Group names and permission codenames
GROUPS = [
    {
        'name': 'executives',
        'permissions': [
            'add_groupobjectpermission',
            'delete_groupobjectpermission',
        ]
    },
]

def add_perms(apps, schema_editor):
    Group = apps.get_model('auth', 'Group')
    Permission = apps.get_model('auth', 'Permission')

    # Add permissions
    for group_dict in GROUPS:
        group, created = Group.objects.get_or_create(name=group_dict['name'])

        for perm_codename in group_dict['permissions']:
            p = Permission.objects.get(codename=perm_codename)
            group.permissions.add(p)

def remove_perms(apps, schema_editor):
    Group = apps.get_model('auth', 'Group')
    Permission = apps.get_model('auth', 'Permission')

    # Remove permissions
    for group_dict in GROUPS:
        try:
            group = Group.objects.get(name=group_dict['name'])
        except Group.DoesNotExist:
            print("Error getting group {0}, skipping permission removal" \
                .format(group_dict['name']))
            break

        for perm_codename in group_dict['permissions']:
            p = Permission.objects.get(codename=perm_codename)
            group.permissions.remove(p)

class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0010_initial_group_data'),
        ('guardian', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(add_perms, remove_perms)
    ]
