# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-11-02 19:14
from __future__ import unicode_literals

from django.db import migrations

# Gives users table-level permissions to t90 grbevents.
# List is based on what's currently in the production database (27 Oct 2017)

# List of usernames
USERS = [
    'robert.coyne@LIGO.ORG',
    'alexander.urban@LIGO.ORG',
    'branson.stephens@LIGO.ORG',
    'chance.norris@LIGO.ORG',
    'xingjiang.zhu@LIGO.ORG',
    'dipongkar.talukder@LIGO.ORG',
    'eric.howell@LIGO.ORG',
    'mark.poe@LIGO.ORG',
    'david.coward@LIGO.ORG',
]

def add_perms(apps, schema_editor):
    User = apps.get_model('auth', 'User')
    Permission = apps.get_model('auth', 'Permission')

    t90_perm = Permission.objects.get(codename='t90_grbevent')
    for username in USERS:
        user, created = User.objects.get_or_create(username=username)

        # Add permission
        user.user_permissions.add(t90_perm)

def remove_perms(apps, schema_editor):
    User = apps.get_model('auth', 'User')
    Permission = apps.get_model('auth', 'Permission')

    t90_perm = Permission.objects.get(codename='t90_grbevent')
    for username in USERS:
        try:
            user = User.objects.get(username=username)
        except User.DoesNotExist:
            print('User {0} not found to remove permissions, skipping' \
                .format(username))
            break

        # Remove permission
        if t90_perm in user.user_permissions.all():
            user.user_permissions.remove(t90_perm)

class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0012_create_custom_permissions'),
    ]

    operations = [
        migrations.RunPython(add_perms, remove_perms)
    ]
