# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-04-02 21:21
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations
from django.contrib.auth.management import create_permissions


GROUP_NAME = settings.EM_ADVOCATE_GROUP
PERMISSION_KWARGS = {
    'content_type__app_label': 'events',
    'codename': 'manage_pipeline',
}



# We have to run this to force the permissions to actually be created.
# Otherwise they are created by a post-migrate signal and it's not possible
# to run all of the migrations in a single command
def create_perms(apps, schema_editor):
    for app_config in apps.get_app_configs():
        app_config.models_module = True
        create_permissions(app_config, apps=apps, verbosity=0)
        app_config.models_module = None


def add_perm(apps, schema_editor):
    Group = apps.get_model('auth', 'Group')
    Permission = apps.get_model('auth', 'Permission')

    # Get permission
    perm = Permission.objects.get(**PERMISSION_KWARGS)

    # Get group
    group = Group.objects.get(name=GROUP_NAME)

    # Add permission to group
    perm.group_set.add(group)


def remove_perm(apps, schema_editor):
    Group = apps.get_model('auth', 'Group')
    Permission = apps.get_model('auth', 'Permission')

    # Get permission
    perm = Permission.objects.get(**PERMISSION_KWARGS)

    # Get group
    group = Group.objects.get(name=GROUP_NAME)

    # Remove permission from group
    perm.group_set.remove(group)


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0022_populate_raven_users_group'),
        ('events', '0033_pipelinelog_and_pipeline_enabled'),
    ]

    operations = [
        migrations.RunPython(create_perms, migrations.RunPython.noop),
        migrations.RunPython(add_perm, remove_perm),
    ]
