# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2018-04-25 01:51
from __future__ import unicode_literals

from django.db import migrations

OLD_ACCOUNTS = {
    'gdb-processor': ['CWB', 'Fermi', 'gstlal', 'HardwareInjection', 'SNEWS',
        'Swift'],
    'gracedb.processor': ['CWB2G', 'Fermi', 'gstlal', 'SNEWS', 'Swift'],
}

NEW_ACCOUNT_USERNAME = 'emfollow'
NEW_ACCOUNT_PIPELINES = list(set([item for items in OLD_ACCOUNTS.values() for
    item in items]))


def update_perms(apps, schema_editor):
    LocalUser = apps.get_model('ligoauth', 'LocalUser')
    UserObjectPermission = apps.get_model('guardian', 'UserObjectPermission')
    Pipeline = apps.get_model('events', 'Pipeline')
    Permission = apps.get_model('auth', 'Permission')
    ContentType = apps.get_model('contenttypes', 'ContentType')

    perm = Permission.objects.get(codename='populate_pipeline')
    ctype = ContentType.objects.get_for_model(Pipeline)

    # Remove UOPs from old accounts which allow pipeline population
    for username, pipelines in OLD_ACCOUNTS.iteritems():
        user = LocalUser.objects.get(username=username)
        for pipeline_name in pipelines:
            pipeline = Pipeline.objects.get(name=pipeline_name)
            uop = UserObjectPermission.objects.get(user=user,
                permission=perm, content_type=ctype, object_pk=pipeline.id)
            uop.delete()

    # Add these UOPs to new acccount
    user = LocalUser.objects.get(username=NEW_ACCOUNT_USERNAME)
    for pipeline_name in NEW_ACCOUNT_PIPELINES:
        pipeline = Pipeline.objects.get(name=pipeline_name)
        uop = UserObjectPermission.objects.create(user=user, permission=perm,
            content_type=ctype, object_pk = pipeline.id)


def revert_perms(apps, schema_editor):
    LocalUser = apps.get_model('ligoauth', 'LocalUser')
    UserObjectPermission = apps.get_model('guardian', 'UserObjectPermission')
    Pipeline = apps.get_model('events', 'Pipeline')
    Permission = apps.get_model('auth', 'Permission')
    ContentType = apps.get_model('contenttypes', 'ContentType')

    perm = Permission.objects.get(codename='populate_pipeline')
    ctype = ContentType.objects.get_for_model(Pipeline)

    # Re-add UOPs to old accounts which allow pipeline population
    for username, pipelines in OLD_ACCOUNTS.iteritems():
        user = LocalUser.objects.get(username=username)
        for pipeline_name in pipelines:
            pipeline = Pipeline.objects.get(name=pipeline_name)
            uop = UserObjectPermission.objects.create(user=user,
                permission=perm, content_type=ctype, object_pk = pipeline.id)

    # Remove these UOPs from new acccount
    user = LocalUser.objects.get(username=NEW_ACCOUNT_USERNAME)
    for pipeline_name in NEW_ACCOUNT_PIPELINES:
        pipeline = Pipeline.objects.get(name=pipeline_name)
        uop = UserObjectPermission.objects.get(user=user, permission=perm,
            content_type=ctype, object_pk = pipeline.id)
        uop.delete()


class Migration(migrations.Migration):

    dependencies = [
        ('guardian', '0002_authorize_users_to_populate_pipelines'),
        ('ligoauth', '0005_update_emfollow_accounts'),
    ]

    operations = [
        migrations.RunPython(update_perms, revert_perms),
    ]
